@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -9,9 +9,10 @@
 #
 package Config::Model;
 {
-  $Config::Model::VERSION = '1.260';
+  $Config::Model::VERSION = '2.008';
 }
 use Any::Moose ;
+use namespace::autoclean;
 use Any::Moose '::Util::TypeConstraints';
 use Any::Moose 'X::StrictConstructor' ;
 
@@ -95,7 +96,7 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 1.260
+version 2.008
 
 =head1 SYNOPSIS
 
@@ -2312,69 +2313,7 @@ sub list_one_class_element {
     return $res ;
 }
 
-=head1 Available models
 
-Returns an array of 3 hash refs:
-
-=over 
-
-=item *
-
-category (system or user or application) => application list. E.g. 
-
- { system => [ 'popcon' , 'fstab'] }
-
-=item *
-
-application => { model => 'model_name', ... }
-
-=item *
-
-application => model_name
-
-=back
-
-=cut
-
-sub available_models {
-   
-    my $path = $INC{"Config/Model.pm"} ;
-    $path =~ s/\.pm// ;
-    my (%categories, %appli_info, %applications ) ;
-
-    get_logger("Model")->trace("available_models: path is $path");
-    foreach my $dir (glob("$path/*.d")) {
-        my ($cat) = ( $dir =~ m!.*/([\w\-]+)\.d! );
-
-        if ($cat !~ /^user|system|application$/) {
-            warn "available_models: skipping unexpected category: $cat\n";
-            next;
-        }
-        
-        get_logger("Model")->trace("available_models: category dir $dir");
-        
-        foreach my $file (sort glob("$dir/*")) {
-            next if $file =~ m!/README! ;
-            my ($appli) = ($file =~ m!.*/([\w\-]+)! );
-            get_logger("Model")->debug("available_models: opening file $file");
-            open (F, $file) || die "Can't open file $file:$!" ;
-            while (<F>) {
-                chomp ;
-                s/^\s+// ;
-                s/\s+$// ;
-                s/#.*// ;
-                my ($k,$v) = split /\s*=\s*/ ;
-                next unless $v ;
-                push @{$categories{$cat}} , $appli if $k =~ /model/i;
-                $appli_info{$appli}{$k} = $v ; 
-                $applications{$appli} = $v if $k =~ /model/i; 
-            }
-        }
-    }
-    return \%categories, \%appli_info, \%applications ;
-}
-
-no Any::Moose ;
 __PACKAGE__->meta->make_immutable ;
 
 1;
@@ -2415,7 +2354,7 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 =head1 LICENSE
 
-    Copyright (c) 2005-2011 Dominique Dumont.
+    Copyright (c) 2005-2012 Dominique Dumont.
 
     This file is part of Config-Model.
 
@@ -2474,7 +2413,7 @@ L<Config::Model::WarpedNode> <- L<Config::Model::AnyThing>
 
 =head2 command line
 
-L<config-edit>
+L<cme>. L<config-edit> is now deprecated.
 
 =head2 Read and write backends
 
@@ -2532,6 +2471,10 @@ L<Config::Model::IdElementReference>
 
 =item *
 
+L<Config::Model::Iterator>
+
+=item *
+
 L<Config::Model::Loader>
 
 =item *
@@ -2576,4 +2519,14 @@ L<Config::Model::Warper>
 
 =back
 
+=head2 Test framework
+
+=over
+
+=item *
+
+L<Config::Model::Tester>
+
+=back
+
 =cut
