@@ -1,18 +1,19 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::FuseUI ;
-BEGIN {
-  $Config::Model::FuseUI::VERSION = '1.226';
+{
+  $Config::Model::FuseUI::VERSION = '2.008';
 }
 
-use Moose ;
+# there's no Singleton with Mouse
+use Any::Moose ;
 
 use Fuse qw(fuse_get_context);
 use Fcntl ':mode';
@@ -20,7 +21,6 @@ use POSIX qw(ENOENT EISDIR EINVAL);
 use Log::Log4perl qw(get_logger :levels);
 use English qw( -no_match_vars ) ;
 
-use MooseX::Singleton;
 has model         => ( is => 'rw', isa => 'Config::Model');
 has root          => ( is => 'ro', isa => 'Config::Model::Node', required => 1 );
 has mountpoint    => ( is => 'ro', isa => 'Str'          , required => 1 );
@@ -30,27 +30,41 @@ my $logger = get_logger("FuseUI") ;
 our $fuseui ;
 
 sub BUILD {
-    $fuseui = shift ; # store singleton object in global variable
+    my $self = shift ;
+    croak (__PACKAGE__," singleton constructed twice" )
+        if defined $fuseui and $fuseui ne $self;
+    $fuseui = $self ; # store singleton object in global variable
 }
 
 # nodes, list and hashes are directories
 sub getdir {
     my $name = shift ;
-    $logger->debug(__PACKAGE__."::getdir called with $name");
+    $logger->debug("FuseUI getdir called with $name");
 
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1, autoadd=>0) ;
     return -EINVAL() unless (ref $obj and $obj->can('children')) ;
     
     my @c = ('..','.', $obj->children ) ;
-    $logger->debug(__PACKAGE__."::getdir return @c , wantarray is ".(wantarray ? 1 : 0) );
+    $logger->debug("FuseUI getdir return @c , wantarray is ".(wantarray ? 1 : 0) );
     return ( @c , 0 ) ;
 }
 
 my %files ;
 
+sub fetch_as_line {
+    my $obj = shift ; 
+    my $v = $obj->fetch(check => 'no') ;
+    $v = '' unless defined $v ;
+    # let's append a \n so that returned files always have a line ending
+    $v .= "\n" unless $v =~ /\n$/ ;
+
+    return $v ;
+}
+
+
 sub getattr {
     my $name = shift ;
-    $logger->debug(__PACKAGE__."::getattr called with $name");
+    $logger->debug("FuseUI getattr called with $name");
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1, autoadd=>0) ;
 
     return -&ENOENT() unless ref $obj ;
@@ -60,9 +74,13 @@ sub getattr {
     # return -ENOENT() unless exists($files{$file});
 
     my $size ;
-    if    ($type eq 'leaf') { $size = length ($obj->fetch || '') ;}
-    elsif ($type eq 'check_list') { $size = length ($obj->fetch || '') ;}
-    else {my @c = $obj->children ; $size = @c; }
+    if ($type eq 'leaf' or $type eq 'check_list') { 
+        $size = length (fetch_as_line($obj)) ;
+    }
+    else {
+        my @c = $obj->children ; 
+        $size = @c; 
+    }
     
     my $mode ;
     if ($type eq 'leaf' or $type eq 'check_list') {
@@ -79,7 +97,7 @@ sub getattr {
 	#return -ENOENT(); # or any other error you care to
 	#print(join(",",($dev,$ino,$modes,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)),"\n");
     my @r = ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks);
-    $logger->trace(__PACKAGE__."::getattr returns '".join("','",@r)."'");
+    $logger->trace("FuseUI getattr returns '".join("','",@r)."'");
 
     return @r ;
 }
@@ -88,13 +106,13 @@ sub getattr {
 sub open {
     # VFS sanity check; it keeps all the necessary state, not much to do here.
     my $name = shift ;
-    $logger->debug(__PACKAGE__."::open called on $name");
+    $logger->debug("FuseUI open called on $name");
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1) ;
     my $type = $obj->get_type ;
     
     return -ENOENT() unless defined $obj;
     return -EISDIR() unless ($type eq 'leaf' or $type eq 'check_list') ;
-    $logger->debug(__PACKAGE__."::open on $name ok");
+    $logger->debug("FuseUI open on $name ok");
     return 0;
 }
 
@@ -103,13 +121,13 @@ sub read {
     # give a byte (ascii "0") to the reading program)
     my ($name,$buf,$off) = @_;
  
-    $logger->debug(__PACKAGE__."::read called on $name");
+    $logger->debug("FuseUI read called on $name");
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1) ;
     my $type = $obj->get_type ;
 
     return -ENOENT() unless defined $obj;
     return -EISDIR() unless ($type eq 'leaf' or $type eq 'check_list') ;
-    my $v = $obj->fetch ;
+    my $v = fetch_as_line($obj) ;
 
     if(not defined $v) {
 	return -EINVAL() if $off > 0;
@@ -119,24 +137,24 @@ sub read {
     return -EINVAL() if $off > length($v);
     return 0 if $off == length($v);
     my $ret = substr($v,$off,$buf); 
-    $logger->debug(__PACKAGE__."::read returns '$ret'");
-    return $ret ;
+    $logger->debug("FuseUI read returns '$ret'");
+    return "$ret" ;
 }
 
 sub truncate {
     my ($name,$off) = @_;
 
-    $logger->debug(__PACKAGE__."::truncate called on $name with length $off");
+    $logger->debug("FuseUI truncate called on $name with length $off");
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1) ;
     my $type = $obj->get_type ;
 
     return -ENOENT() unless defined $obj;
     return -EISDIR() unless ($type eq 'leaf' or $type eq 'check_list') ;
 
-    my $v = substr $obj->fetch, 0, $off ;
+    my $v = substr fetch_as_line($obj) , 0, $off ;
 
-    $logger->debug(__PACKAGE__."::truncate stores '$v'");
-    $obj->store(value => $v, check => 'skip') ;
+    $logger->debug("FuseUI truncate stores '$v'");
+    $obj->store(value => $v, check => 'no') ;
     return 0 ;
 }
 
@@ -146,7 +164,7 @@ sub write {
     if ($logger->is_debug) {
         my $str = $buf ;
         $str =~ s/\n/\\n/g;
-        $logger->debug(__PACKAGE__."::write called on $name with '$str' offset $off");
+        $logger->debug("FuseUI write called on $name with '$str' offset $off");
     }
     
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1) ;
@@ -155,12 +173,12 @@ sub write {
     return -ENOENT() unless defined $obj;
     return -EISDIR() unless ($type eq 'leaf' or $type eq 'check_list') ;
 
-    my $v = $obj->fetch || '';
-    $logger->debug(__PACKAGE__."::write starts with '$v'");
+    my $v = fetch_as_line($obj);
+    $logger->debug("FuseUI write starts with '$v'");
 
     substr $v,$off,length($buf),$buf ;
     chomp $v unless ($type eq 'leaf' and $obj->value_type eq 'string') ;
-    $logger->debug(__PACKAGE__."::write stores '$v'");
+    $logger->debug("FuseUI write stores '$v'");
     $obj->store(value => $v, skip => 'check' ) ;
     return length($buf) ;
 }
@@ -170,11 +188,11 @@ sub mkdir {
     # give a byte (ascii "0") to the reading program)
     my ($name,$mode) = @_;
  
-    $logger->debug(__PACKAGE__."::mkdir called on $name with mode $mode");
+    $logger->debug("FuseUI mkdir called on $name with mode $mode");
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1) ;
     return -ENOENT() unless defined $obj;
 
-    my $type = $obj->get_container_type ;
+    my $type = $obj->container_type ;
     return -ENOENT() unless ($type eq 'list' or $type eq 'hash') ;
 
     return 0 ;
@@ -185,20 +203,20 @@ sub rmdir {
     # give a byte (ascii "0") to the reading program)
     my ($name) = @_;
  
-    $logger->debug(__PACKAGE__."::rmdir called on $name");
+    $logger->debug("FuseUI rmdir called on $name");
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1, autoadd=>0) ;
     return -ENOENT() unless defined $obj;
 
     my $type = $obj->get_type ;
     return -ENOENT() if ($type eq 'leaf' or $type eq 'check_list') ;
 
-    my $ct = $obj->get_container_type ;
+    my $ct = $obj->container_type ;
     my $elt_name = $obj->element_name ;
     my $parent = $obj->parent ;
     
     if ($ct eq 'list' or $ct eq 'hash') {
         my $idx = $obj->index_value ;
-        $logger->debug(__PACKAGE__."::rmdir actually deletes $idx");
+        $logger->debug("FuseUI rmdir actually deletes $idx");
         $parent->fetch_element($elt_name)->delete($idx) ;
     }
     
@@ -210,20 +228,20 @@ sub rmdir {
 sub unlink {
     my ($name) = @_;
 
-    $logger->debug(__PACKAGE__."::unlink called on $name");
+    $logger->debug("FuseUI unlink called on $name");
     my $obj = $fuseui->root->get(path => $name, check => 'skip', get_obj => 1, autoadd=>0) ;
     my $type = $obj->get_type ;
 
     return -ENOENT() unless defined $obj;
     return -EISDIR() unless ($type eq 'leaf' or $type eq 'check_list') ;
     
-    my $ct = $obj->get_container_type ;
+    my $ct = $obj->container_type ;
     my $elt_name = $obj->element_name ;
     my $parent = $obj->parent ;
     
     if ($ct eq 'list' or $ct eq 'hash') {
         my $idx = $obj->index_value ;
-        $logger->debug(__PACKAGE__."::unlink actually deletes $idx");
+        $logger->debug("FuseUI unlink actually deletes $idx");
         $parent->fetch_element($elt_name)->delete($name) ;
     }
     
@@ -260,7 +278,7 @@ Config::Model::FuseUI - Fuse virtual file interface for Config::Model
 
 =head1 VERSION
 
-version 1.226
+version 2.008
 
 =head1 SYNOPSIS
 
@@ -347,4 +365,4 @@ unmounted (with C<fusermount -u mount_point> command)
 
 =head1 SEE ALSO
 
-L<Fuse>, L<Config::Model>
\ No newline at end of file
+L<Fuse>, L<Config::Model>
