@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -14,9 +14,9 @@
       'lircrc',
       {
         'value_type' => 'uniline',
-        'default' => '/etc/lircrc.lcdproc',
+        'upstream_default' => '~/.lircrc',
         'type' => 'leaf',
-        'description' => 'Specify an alternative location of the lircrc file [default: ~/.lircrc]'
+        'description' => 'Specify an alternative location of the lircrc file '
       },
       'prog',
       {
